// readpixel.cpp
// by Glenn G. Chappell
// October 2003
//
// For CS 381
// Example pixel-reading program

#include <iostream>
using std::cerr;
using std::endl;
#include <string>
using std::string;
#include <stdlib.h>  // Some versions of MS-Vis C++ have broken <cstdlib>
//using std::exit;
#include <GL/glut.h> // GLUT stuff, includes OpenGL headers as well


// Global variables
// Window/viewport
const int startwinsize = 400; // Starting window width & height, in pixels
int winw, winh;               // Current width, height, in pixels

// Keyboard
const int ESCKEY = 27;        // ASCII value of escape character

// Color
GLfloat thecolor[3] = { 0.3, 0.3, 0.3 };


// readapixel
// Reads pixel at mouse position (given, GLUT format)
// Puts color in thecolor (global GLfloat [3]).
// Assumes global winh holds height of window/viewport, in pixels.
void readapixel(int x, int y)
{
   glReadPixels(x, winh-y,  // Read pixel at mouse position
                1, 1,       // 1 x 1 block of pixels (single pixel)
                GL_RGB,     // Get R, G, B
                GL_FLOAT,   // Return GLfloat, value 0..1
                thecolor);  // Place to put the result
}


// printbitmap
// Prints the given string at the given raster position
//  using GLUT bitmap fonts.
// You probably don't want any rotations in the model/view
//  transformation when calling this function.
void printbitmap(const string msg, double x, double y)
{
   glRasterPos2d(x, y);
   for (string::const_iterator ii = msg.begin();
        ii != msg.end();
        ++ii)
   {
      glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *ii);
   }
}


// display
// The GLUT display function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT);

   // Draw a random-ish scene with lots of colors
   //  in the left half of the viewport
   glColor3d(0.3, 0.2, 0.3);
   glRectd(-1.0, -1.0, -0.2, 0.4);
   
   glColor3d(0.4, 0.6, 0.9);
   glRectd(-0.9, -0.7, -0.5, 0.2);

   glColor3d(0.9, 0.0, 0.1);
   glRectd(-0.6, -0.8, -0.1, -0.4);

   glColor3d(1.0, 0.8, 0.2);
   glRectd(-0.9, 0.5, -0.1, 0.7);

   glColor3d(0.4, 0.8, 0.4);
   glRectd(-0.7, -0.2, -0.4, 1.0);

   glColor3d(0.8, 0.6, 0.4);
   glRectd(-0.8, -0.9, -0.4, -0.5);

   // Write instructions in drawing window
   glColor3d(0., 0., 0.);
   printbitmap("Click somewhere!", -0.95, 0.8);
   
   // Draw a solid color in the right half of the viewport
   glColor3fv(thecolor);
   glRectd(0.0, -100.0, 100.0, 100.0);

   glutSwapBuffers();
}


// mouse
// The GLUT mouse function
void mouse(int button, int state, int x, int y)
{
   readapixel(x, y);  // Read & store pixel color
   glutPostRedisplay();
}


// motion
// The GLUT motion function
void motion(int x, int y)
{
   readapixel(x, y);  // Read & store pixel color
   glutPostRedisplay();
}


// reshape
// The GLUT reshape function
void reshape(int w, int h)
{
   glViewport(0, 0, w, h);
   winw = w;  // Save window size, "for readapixel" function
   winh = h;

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   // We set up coordinate system so that aspect ratios are always correct,
   //  and the region from -1..1 in x & y always just fits in the viewport.
   if (w > h)
   {
      gluOrtho2D(-double(w)/h, double(w)/h, -1., 1.);
   }
   else
   {
      gluOrtho2D( -1., 1., -double(h)/w, double(h)/w);
   }

   glMatrixMode(GL_MODELVIEW);  // Always go back to model/view mode
}


// keyboard
// The GLUT keyboard function
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
   case ESCKEY:  // ESC: Quit
      exit(0);
      break;
   }
}


// idle
// The GLUT idle function
void idle()
{
   // Print OpenGL errors, if there are any (for debugging)
   if (GLenum err = glGetError())
   {
      cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
   }
}


// init
// Initializes GL states
// Called by main after window creation
void init()
{
   glClearColor(1.0, 1.0, 1.0, 0.0);
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("CS 381 - Read Pixels Demo");

   // Initialize GL states & register callbacks
   init();
   glutDisplayFunc(display);
   glutMouseFunc(mouse);
   glutMotionFunc(motion);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   glutIdleFunc(idle);

   // Do something
   glutMainLoop();

   return 0;
}
